package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import java.util.Date;
import java.util.List;

import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordDataIdsReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdateExamRecordDataBatchNumReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordDataIdsResp;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;

/**
 * @Description 获取待处理数据
 * @Author xt
 * @Date 2019/12/17 16:39
 * @Version 1.0
 */
@Component
public class DataGainExamExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    private final static Integer batchSize = 200;

    private final static Long batchNum = new Date().getTime();

    @Autowired
    private ExamRecordDataCloudService examRecordDataCloudService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Override
    public void execute(Long key, ExamRecordData value, List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {
        // 获取考试信息id
        Long startId = 0l;
        GetExamRecordDataIdsReq req = new GetExamRecordDataIdsReq();
        req.setBatchNum(batchNum);
        req.setSize(batchSize);
        for (;;) {
            req.setStartId(startId);
            GetExamRecordDataIdsResp res = examRecordDataCloudService.getExamRecordDataIds(req);
            List<Long> ids = res.getExamRecordDataIds();
            if (ids == null || ids.size() == 0) {
                return;
            }
            for (Long id : ids) {
                // 根据id获取考试信息缓存
                ExamRecordData erd = examRecordDataService.getExamRecordDataCache(id);
                if (erd == null) {
                    throw new StatusException("1001", "获取Redis中考试信息为空");
                }
                outList.add(new KeyValuePair<Long, ExamRecordData>(id, erd));
            }

            // 修改已获取过的考试信息batchNum
            UpdateExamRecordDataBatchNumReq ureq = new UpdateExamRecordDataBatchNumReq();
            ureq.setBatchNum(batchNum);
            ureq.setIds(ids);
            examRecordDataCloudService.updateExamRecordDataBatchNum(ureq);
            startId = ids.get(ids.size() - 1);
        }
    }

}
